// SedMul.c 250703  Stadtmueller    mst3    230703
//      SedPc & Sed1Pc & Sed2Pc from Version 98
//
//      SedPut, SedGet
// multi telegram tranfer routines
//

#include <stdlib.h>
#include "SedPc.h"

//-------+---------+---------+---------+---------+---------+---------+---------+

// SedPut   put an array with read or write information to sedac
//
//	5 bytes per telegram are send to the SedPc-fifo.
//		function/crateaddr, subaddr, msb-data, lsb-data, end-byte
//
//  call: SedPut (lLn, lplNum, lrglArr, lplErr);
//  para:   lLn   = logical line-number for that block
//          wNum  = number of telegrams in array, if error
//                    wNum is telegram-number when error had occured
//                    max. array-size is size of FIFO
//    if write
//       wArr(n)  = cn ; bits : 7..6=10(rd) or =01(wr), 5=cp, 4..0=cn
//       wArr(n+1)= sa ; bits : 7..0 subaddress
//       wArr(n+2)= dat; bits : 15..0 data to sedac
//    if read
//       wArr(n)  = cn
//       wArr(n+1)= sa
//       wArr(n+2)= dat; bits : 15..0 data to sedac (will be ignored in SedPc)
//
//          *lplErr = 0 = no error, exit after error
//            bit 13 = illegal SEDPC type
//            bit 12 = no such line-number
//            bit  9 = Reserve / Release Error, wrong Handle
//            bit  8 = Reserve / Release Error, wrong Count
//            bit  7 = ParameterError: one parameter incorrect
//            bit  6 = Response/Busy not set; data not taken by SEDPC
//            bit  3 = LastByte marker not set or illegal function
//  subp: sedsel, sedwr0, sedwr4

void SedPut ( LONG lLn, LONG *lplNum, LONG *lrglArr, LONG *lplErr )

{
    static LONG	err, i, fadd, sa, fcn;
    static LONG	*p;

    if ( *lplErr = sedsel(lLn) )    // set device-addresses to externals
        goto exit;

    p = lrglArr;                    // pointer to array

    for ( i = 0; i < *lplNum; i++ ) {   // get telegrams from array
        fcn = *(p++);                   // get function and crate-number
        sa = *(p++);                    // get subaddress
        if ( fcn & 0xFF00 || sa & 0xFF00 )  // check fcn and sa
            goto err7;

        fadd = fcn & 0xC0;
        if ( fadd == 0x80 || fadd == 0x40 ) {   // function = read or write ?
            fadd = fcn << 8 | sa;
            *lplErr  = sedwr4 (fadd, *(p++));   // ..to SedPc
            *lplErr |= sedwr0 (LASTBY);         // LastByteMarker to SedPc
            if ( *lplErr )
                goto err;
        } else
            goto err3;                  // illegal function
    }

exit:
    return;

err7:
    *lplErr = 0x80;             // parameter error
    goto err;
err3:
    *lplErr = 0x8;              // illegal function
err:
    *lplNum = i + 1;

}		// end of SedPut
//-------+---------+---------+---------+---------+---------+---------+---------+

// SedGet   get data and status from sedac and put it into an array
//
//	4 bytes per telegram are received from the SedPc-fifo.
//		status, msb-data, lsb-data, end-byte
///
//  call: sedget (lLn, lplNum, lrglArr, lplErr);
//  para:   lLn   = logical line-number for that block
//          wNum  = number of telegrams expected, if error
//                  wNum is telegram-number when error had occured
//                  max. array-size is size of FIFO
//    if write
//       wArr(n)   = status ; bits : 7..6=10(rd), 5..4=10, 3..0=LASTBY,FE,PE,RE
//       wArr(n+1) = data (don't use it)
//    if read
//       wArr(n)   = status ; bits : 7..6=10(rd), 5..4=10, 3..0=LASTBY,FE,PE,RE
//       wArr(n+1) = data
//               LASTBY = LastByteMarker was not set
//
//          *lplErr = 0 = no errors, exit after error
//            bit 13 = illegal SEDPC type
//            bit 12 = no such line-number
//            bit  9 = Reserve / Release Error, wrong Handle
//            bit  8 = Reserve / Release Error, wrong Count
//            bit  5 = Bytecount mismatch
//            bit  4 = Timeout: no response from SEDPC
//            bit  3 = LastByte marker not set or illegal function
//            bit  2..0 = Code of Sedac Error, OR of all telegrams
//  subp: sedsel, sedrd1, sedrd2

void SedGet ( LONG lLn, LONG *lplNum, LONG *lrglArr, LONG *lplErr )

{
    static LONG	i, dat, err;
    static LONG	*p;

    if ( *lplErr = sedsel(lLn) )    // set device-addresses to externals
        goto exit;

    p = lrglArr;                    // pointer to array
    err = 0;

    for ( i = 0; i < *lplNum; i++ ) {   // get status and data from SedPc
        *lplErr  = sedrd1 (&dat);       // get a status-byte
        *(p++)   = dat;                 // status to array
        if ( (dat&0xF0)==0xA0 || (dat&0xF0)==0x60 )   // rd or wr ?
            err |= dat;
        else
            err |= 0x8 | dat;       // IllegalFunction or ByteCount wrong

        *lplErr |= sedrd2 (p++);    // get data-word
        *lplErr |= sedrd1 (&dat);   // get LastByteMarker
        if ( dat != LASTBY )
            err |= 0x8;             // LastByteMarker not set

        if ( *lplErr )              // CommunicationError with SedPc ?
            goto err;
    }

exit:
    return;

err:
    err |= *lplErr;
    *lplNum = i + 1;
    goto exit;

}		// end of SedGet
//-------+---------+---------+---------+---------+---------+---------+---------+

// end of SedMul.c
